#!/usr/bin/python
#
# GUI for Update Agent
# Copyright (c) 1999, 2000 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>

import sys, os

saveargs = []
for arg in sys.argv[1:]:
    saveargs.append(arg)
    sys.argv.remove(arg)

import gtk, GTK, GDK
from gtk import TRUE
from gtk import FALSE
import gnome, gnome.ui
import libglade
import signal
import rpm

sys.path.append("/usr/share/rhn/up2date")

import progress
import up2date
import checklist

class Gui:

    def __init__(self):
        self.gotPackages = FALSE
        self.gotSkippedPackages = FALSE
        
        self.xml = libglade.GladeXML("/usr/share/rhn/up2date/gui.glade", "mainWin")
        self.xml.signal_autoconnect (
            { "onDruidCancel" : self.onDruidCancel,
              "onStartPageNext" : self.onStartPageNext,
              "onSkippedPagePrepare" : self.onSkippedPagePrepare,
              "onSkippedPageNext" : self.onSkippedPageNext,
              "onPackagePagePrepare" : self.onPackagePagePrepare,
              "onPackagePageNext" : self.onPackagePageNext,
              "onPackagePageBack" : self.onPackagePageBack,
              "onDependencyPagePrepare" : self.onDependencyPagePrepare,
              "onRetrievalPagePrepare" : self.onRetrievalPagePrepare,
              "onRetrievalPageNext" : self.onRetrievalPageNext,
              "onRetrievalPageBack" : self.onRetrievalPageBack,
              "onInstallPagePrepare": self.onInstallPagePrepare,
              "onInstallPageBack": self.onInstallPageBack,
              "onFinishPagePrepare" : self.onFinishPagePrepare,
              "onFinishPageFinish" : self.onFinishPageFinish } )

        self.druid = self.xml.get_widget("druid")
        self.mainWin = self.xml.get_widget("mainWin")
        self.mainWin.connect("delete-event", gtk.mainquit)
        self.mainWin.connect("hide", gtk.mainquit)
        self.setupSkippedList()
        self.setupPackageList()

        # background on finished page
        box = self.xml.get_widget("finishEventBox")
        style = box.get_style().copy()
        color = gtk.GdkColor(52232, 0, 0)
        style.bg[GTK.STATE_NORMAL] = color
        box.set_style(style)
        
        self.mainWin.show_all()
        self.xml.get_widget("packagesFinishedLabel").hide()
        self.xml.get_widget("installingFinishedLabel").hide()
        

    def onDruidCancel(self, dummy):
        gtk.mainquit()


    def fatalError(self, error):
        dlg = gnome.ui.GnomeErrorDialog(error, self.mainWin)
        dlg.run_and_close()
        
        gtk.mainquit()
        sys.exit()


    def setBusyCursor(self):
        cursor = gtk.cursor_new (GDK.WATCH)
        self.mainWin.get_window().set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)


    def setArrowCursor(self):
        cursor = gtk.cursor_new (GDK.LEFT_PTR)
        self.mainWin.get_window().set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)
            

    def updateSelectedSize(self, state, rowData):
        sizeLabel = self.xml.get_widget("sizeLabel")
        hdr = up2date.getHeader(rowData)
        
        if state == 0:
            self.totalSelectedSize = self.totalSelectedSize - hdr['size']
        else:
            self.totalSelectedSize = self.totalSelectedSize + hdr['size']
        sizeLabel.set_text("%d kB" % (self.totalSelectedSize / 1024))
        if self.totalSelectedSize > 0 and not self.pkgsSelected:
            self.druid.set_buttons_sensitive(TRUE, TRUE, TRUE)
            self.pkgsSelected = TRUE
        elif self.totalSelectedSize == 0 and self.pkgsSelected:
            self.druid.set_buttons_sensitive(TRUE, FALSE, TRUE)
            self.pkgsSelected = FALSE
            
    def setupSkippedList(self):
        window = self.xml.get_widget("skippedWindow")
        
        self.skippedArea = checklist.CheckList(3)
        self.skippedArea.set_column_title(1, "Package Name")
        self.skippedArea.set_column_min_width(1, 200)
        self.skippedArea.set_column_title(2, "Version")
        self.skippedArea.set_column_title(3, "Release")
        self.skippedArea.column_titles_show()

        self.skippedArea.connect("select_row", self.showPackageInfo,
                                 (self.xml.get_widget("skippedPackageInfoLabel")))
        window.add(self.skippedArea)

        
    def setupPackageList(self):
        window = self.xml.get_widget("packageWindow")
        
        self.packageArea = checklist.CheckList(3)
        self.packageArea.set_column_title(1, "Package Name")
        self.packageArea.set_column_min_width(1, 200)
        self.packageArea.set_column_title(2, "Version")
        self.packageArea.set_column_title(3, "Release")
        self.packageArea.column_titles_show()
        self.packageArea.set_toggled_func(self.updateSelectedSize)

        self.packageArea.connect("select_row", self.showPackageInfo,
                                 (self.xml.get_widget("packageInfoLabel")))
        window.add(self.packageArea)


    def showPackageInfo(self, area, row, column, event, label):
        rowData = area.get_row_data(row)
        hdr = up2date.getHeader(rowData[1])
        label.set_text(hdr['summary'] + "\n" + hdr['description'])


    def onStartPageNext(self, page, dummy):
        if not self.gotPackages:
            self.pkgList = []
            self.selectedPkgList = []
            self.skipPkgList = []
            self.pkgsSelected = FALSE
            self.totalSelectedSize = long(0)
            self.druid.set_buttons_sensitive(TRUE, FALSE, TRUE)
            
            pwin = progress.Progress()
            pwin.setLabel("Red Hat Update Agent is building a list of updated\nRPM packages installed on your system.  Please wait.")
            
            try:
                (self.pkgList, self.skipPkgList) = up2date.getUpdatedPackageList(pwin.setLabel, pwin.setProgress)
            except up2date.CommunicationError, e:
                pwin.hide()
                self.fatalError("There was a fatal error communicating with the server.  The message was:\n" + e.errmsg)

            pwin.hide()
            while gtk.events_pending():
                gtk.mainiteration(FALSE)

            self.gotPackages = TRUE

        if len(self.skipPkgList) > 0:
            self.druid.set_page(self.xml.get_widget("skippedPage"))
        else:
            self.druid.set_page(self.xml.get_widget("packagePage"))
            
        return TRUE

    def onSkippedPagePrepare(self, page, dummy):
        if not self.gotSkippedPackages:
            self.skippedArea.freeze()
            for pkg in self.skipPkgList:
                self.skippedArea.append_row((pkg[0], pkg[1], pkg[2]),
                                            FALSE, pkg)
            self.skippedArea.thaw()

            self.gotSkippedPackages = TRUE

    def onSkippedPageNext(self, page, dummy):
        pass
        # build list of selected skipped packages
        #rowData = ()
        #for index in range(self.skippedArea.n_rows):
        #    rowData = self.skippedArea.get_row_data(index)
        #    if rowData[0] == 1:
        #        if rowData[1] not in self.pkgList:
        #            self.pkgList.append(rowData[1])
        #    elif rowData[0] == 0:
        #        if rowData[1] in self.pkgList:
        #            self.pkgList.remove(rowData[1])

        #self.pkgList.sort()
        
        #return FALSE

    
    def onPackagePagePrepare(self, page, dummy):
        self.packageArea.freeze()
        self.packageArea.clear()
        self.totalSelectedSize = long(0)

        if len(self.skipPkgList) > 0:
            for index in range(self.skippedArea.n_rows):
                rowData = self.skippedArea.get_row_data(index)
                if rowData[0] == 1:
                    self.packageArea.append_row((rowData[1][0],
                                                rowData[1][1],
                                                rowData[1][2]),
                                                TRUE, rowData[1])
                    self.totalSelectedSize = self.totalSelectedSize + up2date.getHeader(rowData[1])['size']
        elif len(self.pkgList) == 0:
            dlg = gnome.ui.GnomeOkDialog("Your system is fully updated.  No new packages are needed.", self.mainWin)
            dlg.run_and_close()
            sys.exit(0)
            
        for pkg in self.pkgList:
            self.packageArea.append_row((pkg[0], pkg[1], pkg[2]), FALSE, pkg)

        self.xml.get_widget("sizeLabel").set_text("%d kB" % (self.totalSelectedSize / 1024))
        self.packageArea.thaw()
        

    def onPackagePageNext(self, page, dummy):
        def cb(foo):
            return foo
        
        if self.totalSelectedSize > up2date.freeDiskSpace():
            dlg = gnome.ui.GnomeErrorDialog("The packages you have selected total %d kB, but\nYou only have %d kB of free space.\nPlease deselect some packages and continue." % (self.totalSelectedSize / 1024, up2date.freeDiskSpace() / 1024),
                                            self.mainWin)
            return TRUE

        if self.totalSelectedSize == 0:
            dlg = gnome.ui.GnomeErrorDialog("You must select at least one package.", self.mainWin)
            return TRUE

        # build list of selected packages
        rowData = ()
        self.selectedPkgList = []
        newPackages = []
        for index in range(self.packageArea.n_rows):
            rowData = self.packageArea.get_row_data(index)
            if rowData[0] == 1:
                self.selectedPkgList.append(rowData[1])

        pwin = progress.Progress()

        while gtk.events_pending():
            gtk.mainiteration(FALSE)
        try:
            self.depPackages = up2date.dryRun(self.selectedPkgList,
                                              pwin.setLabel,
                                              pwin.setProgress)
        except up2date.CommunicationError, e:
            pwin.hide()
            self.fatalError("There was a fatal error communicating with the server.  The message was:\n" + e.errmsg)
        except up2date.RpmError, e:
            pwin.hide()
            self.fatalError("There was a fatal RPM error.  The message was:\n" + e.errmsg)
        except up2date.DependencyError, e:
            found = 0
            pwin.hide()
            for index in range(self.packageArea.n_rows):
                rowData = self.packageArea.get_row_data(index)
                if rowData[1][0] == e.pkgName:
                    dlg = gnome.ui.GnomeQuestionDialog(e.errmsg + "\n\nAdd %s to selected packages?" % e.pkgName, cb)
                    if dlg.run_and_close():
                        self.fatalError("RPM Dependency error cannot be satisfied.")
                    else:
                        self.selectedPkgList.append(rowData[1])
                    found = 1
                    self.depPackages = []
                    break
            if not found:
                self.fatalError("There was a fatal RPM error.  The message was:\n" + e.errmsg)
            
        pwin.hide()
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        if len(self.depPackages) > 0:
            self.druid.set_page(self.xml.get_widget("dependencyPage"))
            return TRUE
        else:
            self.druid.set_page(self.xml.get_widget("retrievalPage"))
            return TRUE


    def onPackagePageBack(self, page, dummy):
        if len(self.skipPkgList) > 0:
            self.druid.set_page(self.xml.get_widget("skippedPage"))
        else:
            self.druid.set_page(self.xml.get_widget("startPage"))
        return TRUE


    def onDependencyPagePrepare(self, page, dummy):
        area1 = self.xml.get_widget("dependencyArea1")
        area2 = self.xml.get_widget("dependencyArea2")        
        area1.clear()
        area2.clear()
        for pair in self.depPackages:
            area1.append((pair[0][0], pair[0][1], pair[0][2]))
            area2.append((pair[1][0], pair[1][1], pair[1][2]))


    def setProgress(self, amount, total):
        if total != 0:
            i = (amount + 1.0) / total
            if i > 1:
                i = 1.0
            elif i < 0:
                i = 0
        else:
            i = 0

        self.progress.update(i)
        while (gtk.events_pending()):
            gtk.mainiteration(FALSE)

    def doRetrieval(self):

        def cb(arg):
            return arg
        
        gtk.threads_enter()
        self.progress = self.xml.get_widget("retrievePackageProgress")
        totalProgress = self.xml.get_widget("retrieveTotalProgress")
        totalProgress.update(0)
        

        self.setBusyCursor()
        
        pkgCount = len(self.selectedPkgList)
        label = self.xml.get_widget("retrievePackageNameLabel")
        for index in range(pkgCount):
            up2date.getPackage(self.selectedPkgList[index], label.set_text,
                              self.setProgress)
            
            # do signature checking only if GPG installation checks out,
            # and GPG checking is enabled.
            # we already warned them about any errors when we started up.
            if up2date.cfg.readEntry("useGPG") and not up2date.checkGPGInstallation():
                res = up2date.hasBadSignature(self.selectedPkgList[index])
                if res == 1:
                    dlg = gnome.ui.GnomeQuestionDialog("The package %s is not signed with a GPG signature.  Continue?" % up2date.pkgToString(self.selectedPkgList[index]), cb)
                    if dlg.run_and_close():
                        sys.exit(1)
                if res == 2:
                    dlg = gnome.ui.GnomeQuestionDialog("The package %s does not have a valid GPG signature.\nIt has been tampered with or corrupted.  Continue?" % up2date.pkgToString(self.selectedPkgList[index]), cb)
                    if dlg.run_and_close():
                        sys.exit(1)

                
            i = (index + 1.0) / pkgCount
            if i > 1:
                i = 1.0
            elif i < 0:
                i = 0
            totalProgress.update(i)
            while gtk.events_pending():
                gtk.mainiteration(FALSE)

        self.xml.get_widget("packagesFinishedLabel").show()
        self.druid.set_buttons_sensitive(TRUE, TRUE, TRUE)
        # XXX remove me when locking is fixed in gtk
        self.setArrowCursor()
        gtk.threads_leave()
        return 0


    def onRetrievalPagePrepare(self, page, dummy):
        self.druid.set_buttons_sensitive(FALSE, FALSE, TRUE)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        # total hack to get this function to start as soon as this
        # on exits.  GTK makes me do ugly things.
        gtk.timeout_add(0, self.doRetrieval)


    def onRetrievalPageNext(self, page, dummy):
        if up2date.retrieveOnly():
            self.druid.set_page(self.xml.get_widget("finishPage"))
        else:
            self.druid.set_page(self.xml.get_widget("installPage"))
        return TRUE

    def onRetrievalPageBack(self, page, dummy):
        self.druid.set_page(self.xml.get_widget("packagePage"))
        return TRUE
    

    def rpmCallback(self, what, amount, total, hdr, path):
        if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            try:
                fd = os.open(fileName, os.O_RDONLY)
            except:
                raise up2date.RpmError("Error opening %s" % fileName)
            
            return fd
        elif what == rpm.RPMCALLBACK_INST_START:
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            self.pkgIndex = self.pkgIndex + 1
            self.pkgLabel.set_text(fileName)
        elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
            name = hdr['name']
            version = hdr['version']
            release = hdr['release']
            epoch = hdr['epoch']
            if epoch == None:
                epoch = ""
            up2date.remoteAddPackage((name, version, release, epoch))
            
            i = (self.pkgIndex * 1.0) / self.pkgCount
            if i > 1:
                i = 1.0
            elif i < 0:
                i = 0
            self.totalProgress.update(i)

        elif what == rpm.RPMCALLBACK_INST_PROGRESS:
            i = (amount + 1.0) / total
            if i > 1:
                i = 1.0
            elif i < 0:
                i = 0
            self.progress.update(i)
        elif what == rpm.RPMCALLBACK_UNINST_STOP:
            uninstHdr = rpm.getTransactionCallbackHeader()
            name = uninstHdr['name']
            version = uninstHdr['version']
            release = uninstHdr['release']
            epoch = uninstHdr['epoch']
            if epoch == None:
                epoch = ""
            up2date.remoteDelPackage((name, version, release, epoch))

        while gtk.events_pending():
            gtk.mainiteration(FALSE)


    def doInstallation(self):
        gtk.threads_enter()
        self.progress = self.xml.get_widget("installPackageProgress")
        self.totalProgress = self.xml.get_widget("installTotalProgress")
        self.totalProgress.update(0)
        self.pkgLabel = self.xml.get_widget("installPackageNameLabel")
        self.pkgIndex = 0
        self.pkgCount = len(self.selectedPkgList)

        self.setBusyCursor()
        try:
            up2date.installPackages(self.selectedPkgList, self.rpmCallback)
        except up2date.RpmError, e:
            self.setArrowCursor()
            self.fatalError("There was a fatal RPM error.  The message was:\n" + e.errmsg)            

        if not up2date.cfg.readEntry("keepAfterInstall"):
            for pkg in self.selectedPkgList:
                try:
                    up2date.removePackage(pkg)
                except up2date.FileError, e:
                    dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)

                while gtk.events_pending():
                    gtk.mainiteration(FALSE)

        self.xml.get_widget("installingFinishedLabel").show()
        self.druid.set_buttons_sensitive(TRUE, TRUE, TRUE)
        # XXX remove me when locking is fixed in gtk
        self.setArrowCursor()
        gtk.threads_leave()
        return 0
    

    def onInstallPagePrepare(self, page, dummy):
        self.druid.set_buttons_sensitive(FALSE, FALSE, TRUE)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        # total hack to get this function to start as soon as this
        # on exits.  GTK makes me do ugly things.
        gtk.timeout_add(0, self.doInstallation)


    def onInstallPageBack(self, page, dummy):
        self.druid.set_page(self.xml.get_widget("packagePage"))
        return TRUE


    def onFinishPagePrepare(self, page, dummy):
        druid = self.xml.get_widget("druid")
        druid.set_buttons_sensitive(FALSE, TRUE, FALSE)
        druid.set_show_finish(TRUE)
        
        text = "The Red Hat Update Agent has finished "
        if up2date.retrieveOnly():
            text = text + "retrieving\n the following packages successfully:\n\n"
        else:
            text = text + "installing\n the following packages successfully:\n\n"

        for pkg in self.selectedPkgList:
            text = text + up2date.pkgToString(pkg) + "\n"

        if up2date.retrieveOnly():
            text = text + """Because of your settings, these packages were only downloaded,
not installed.  If you wish to change your settings, please run the
"up2date-config" program."""

        self.xml.get_widget("finishArea").insert_defaults(text)
        return FALSE


    def onFinishPageFinish(self, page, dummy):
        gtk.mainquit()
    

def showAbout():
    dlg = gnome.ui.GnomeAbout("About Red Hat Update Agent",
                              "2.0.6",
                              "Copyright (c) 1999, 2000 Red Hat, Inc.",
                              ["Preston Brown <pbrown@redhat.com>"],
                              "This software is distributed under the GPL.  Please Report bugs to Red Hat's Bug Tracking System: http://bugzilla.redhat.com")
    dlg.run_and_close()

def rootWarning():
    dlg = gnome.ui.GnomeErrorDialog("You must run the Update Agent as root.")
    dlg.run_and_close()
    
                            
def main():
    def cb(foo):
        return foo
        
    test = 0

    if len(saveargs) > 0:
        if saveargs[0] == "-t" or saveargs[0] == "--test":
            test = 1
        if saveargs[0] == "-a" or saveargs[0] == "--about":
            showAbout()
            sys.exit(0)
        
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    if not up2date.hasSSL():
        dlg = gnome.ui.GnomeQuestionDialog("""Your system libraries do not support SSL (secure) connections.  Any data
that you send or receive from redhat.com will be transmitted in the clear.

Continue anyway?""", cb)
        if dlg.run_and_close():
            sys.exit(1)

    if up2date.getSystemId() == None:
        dlg = gnome.ui.GnomeQuestionDialog("""You are not registered with Red Hat Network Services.  To use Update Agent,
You must be registered.

However, you can continue now with an anonymous registration, and only
your Red Hat Linux version and computer architecture will be sent to
redhat.com.  You are free to use the rhn_register later to take full
advantage of the benefits offered by the Red Hat Network.

Register anonymously?""", cb)
        if dlg.run_and_close():
            sys.exit(1)
        else:
            try:
                anony = up2date.anonymousRegister()
            except up2date.CommunicationError, e:
                dlg = gnome.ui.GnomeErrorDialog("There was a fatal error communicating with the server.  The message was:\n" + e.errmsg)
                dlg.run_and_close()
                sys.exit(1)

            if anony == 0:
                dlg = gnome.ui.GnomeErrorDialog("There was some sort of error writing your anonymous registration key\nto disk.")
                dlg.run_and_close()
                sys.exit(1)

    if up2date.cfg.readEntry("useGPG") and up2date.checkGPGInstallation() == 1:
        dlg = gnome.ui.GnomeQuestionDialog("""GPG does not appear to be installed.  Without it, you will be unable to
verify that packages Update Agent downloads are securely signed by a
trusted source.

Continue anyway?""", cb)
        if dlg.run_and_close():
            sys.exit(1)

    if up2date.cfg.readEntry("useGPG") and up2date.checkGPGInstallation() == 2:
        dlg = gnome.ui.GnomeQuestionDialog("""Your GPG keyring does not contain the Red Hat, Inc. public key.
Without it, you will be unable to verify that packages Update Agent downloads
are securely signed by Red Hat.

Install key now?""", cb)
        if not dlg.run_and_close():
            if up2date.addGPGKey():
                dlg = gnome.ui.GnomeErrorDialog("Some sort of error occurred adding the Red Hat\nGPG key to your keyring.")
                dlg.run_and_close()
                sys.exit(1)
            
    if os.geteuid() != 0 and not test:
        rootWarning()
        sys.exit(1)

    gui = Gui()
    gtk.mainloop()

    
if __name__ == "__main__":
    try:
        main()
    except IOError, e:
        print "There was some sort of I/O error: %s" % e

